#!/usr/local/bin/perl -w
##########################################################
# UCS Checking Script by David Luyer                     #
##########################################################
# Notes:                                                 #
#        * This only checks that connection is accepted! #
#        * IP addresses used to prevent DNS problems.    #
#        * DNS lookup for invalid.uwa.edu.au hardcoded   #
#          might need changing for use elsewhere         #
#        * Very strange regexp to determine proxy ports  #
#        * Update $DIR below                             #
#        * sock.pl may need OS-specific changes          #
##########################################################

$DIR = "/path/to/checker";
require "flush.pl";
require "$DIR/sock.pl";
require "$DIR/checkerconfig.pl";

sub probe {
local ($handle);

  if($checkerconfig'delay{$host}) {
    $timeout=$checkerconfig'delay{$host};
  } elsif($checkerconfig'delay{"$host:$service"}) {
    $timeout=$checkerconfig'delay{"$host:$service"};
  } elsif($checkerconfig'maindelay) {
    $timeout=$checkerconfig'maindelay;
  } else {
    $timeout=60;
  }
  $handle = 0;
  $error = 0;
  $SIG{'ALRM'} = 'timeout';
  alarm($timeout);
  $srv = $service;
  $srv = $checkerconfig'servicemap{$service}
	if ($checkerconfig'servicemap{$service});
  $handle = &sock'open($checkerconfig'ip{$host}, $srv);
  alarm(0);
  $SIG{'ALRM'} = 'DEFAULT';
  if(!$handle) {
    if(!$error) {
      $error = $!;
    }
    $stdoutbuf .= $error;
    if($checkerconfig'ok{"$host:$service"}) {
      for $err (@{$checkerconfig'ok{"$host:$service"}}) {
        if($err eq $error) {
          $stdoutbuf .= " [OK]\n";
          $errok++;
          return;
        }
      }
    }
    $errbad++, $herrs++;
    #$stdoutbuf .= "\n^^^ ############################## ERROR ############################## ^^^\n";
    $stdoutbuf .= "\n";
    printflush(STDERR,"ERROR [$errbad]: Host: $host ($checkerconfig'ip{$host}) Service: $service Error: $error\n");
    return;
  }
  # ok, now check for a banner
  if($service eq "telnet") {
    print $handle "\377\375\3"; # you supress go ahead (order)
    print $handle "\377\375\1"; # you echo (order)
    print $handle "\377\376 "; # you don't send terminal speed (order)
  } elsif($service eq "www" || $service =~ /^[78][018][028][0248]$/) {
    print $handle "HEAD / HTTP/1.0\r\n\r\n";
  } elsif($service eq "gopher") {
    print $handle "GET 0/error\r\n";
  } elsif($service eq "6667") {
    print $handle "PING invalid\r\n";
  } elsif($service eq "whois") {
    print $handle "invalid-test\r\n";
  } elsif($service eq "domain") {
    print $handle "\0\$";
    print $handle "\32a\1\0\0\1\0\0\0\0\0\0\7invalid\3uwa\3edu\2au\0\0\1\0\1";
  } elsif($service eq "113") {
    print $handle "0 , 0\r\n";
  } elsif($service eq "1080") {
    # SOCKS4, but SOCKS5 will winge = data, too
    print $handle "\4\1\0\27\177\0\0\1root\0";
  }
  flush($handle);
  $timeout *= 4; # Allow 4 times the connect timeout for a banner timeout
  eval {
    local $SIG{ALRM} = sub { die "alarm\n" };
    local $buffer;

    alarm($timeout);
    sysread $handle, $buffer, 128;
    alarm(0);
  };
  $error = $@ if($@ && $@ ne "alarm\n");
  $error = "No banner ($timeout seconds)" if($@ eq "alarm\n");
  if($@) {
    $errbad++, $herrs++;
    &sock'close($handle);
    close($handle);
    $stdoutbuf .= $error;
    #$stdoutbuf .= "\n^^^ ############################## ERROR ############################## ^^^\n";
    $stdoutbuf .= "\n";
    printflush(STDERR,"ERROR [$errbad]: Host: $host ($checkerconfig'ip{$host}) Service: $service Error: $error\n");
  } else {
    $errnone++;
    &sock'close($handle);
    close($handle);
    $stdoutbuf .= "OK\n";
  }
}

sub timeout {

  $error = "Timeout ($timeout seconds)";
}

sub check {
local($servs);

  if($checkerconfig'services{$host}) {
    $stdoutbuf = $host." (".$checkerconfig'ip{$host}.") ";
    $stdoutok = $host." Services: ";
    $herrs = 0;
    $servs = 1;
    $service = "telnet";
    $stdoutok .= $service.", ";
    probe();
    for $service (@{$checkerconfig'services{$host}}) {
      $stdoutbuf .= "**  Service: ".$service." ";
      $stdoutok .= $service.", ";
      probe();
    }
    chop $stdoutok;
    chop $stdoutok;
  } else {
    $stdoutbuf = $host." (".$checkerconfig'ip{$host}.") ";
    $herrs = 0;
    $servs = 0;
    $service = "telnet";
    probe();
  }
  if($herrs) {
    $stdoutstr .= "** Host: ".$stdoutbuf;
  } elsif($servs) {
    $stdoutstr .= ">> Host: ".$stdoutok." [OK]\n";
  } else {
    $stdoutstr2len += length($host) + 2;
    if($stdoutstr2len > 62) {
      $stdoutstr2 .= "\n++        ";
      $stdoutstr2len = length($host) + 2;
    }
    $stdoutstr2 .= $host.", ";
  }
  undef $herrs, $stdoutbuf, $stdoutok;
}

sub run {
  local($msg) = @_;

  $errbad = 0;
  $errok = 0;
  $errnone = 0;
  $stdoutstr = "";
  $stdoutstr2len = 0;
  $stdoutstr2 = "++ Hosts: ";
  printflush(STDERR,"UCS CHECKER STARTING ",`date`);
  for $host (keys %checkerconfig'ip) {
    check();
  }
  chop $stdoutstr2;
  chop $stdoutstr2;
  if(!$errbad) {
    $stdoutstr = "++ Hosts: all [OK]\n";
  } else {
    $stdoutstr .= $stdoutstr2." [OK]\n";
  }
  @pl = ( ($errok == 1) ? "" : "S", ($errbad == 1) ? "" : "S" );
  $stdoutstr .=
	"TOTAL: $errnone OK, $errok ERROR$pl[0] OK, $errbad ERROR$pl[1] BAD.\n";
  print(STDOUT $stdoutstr);
  $errstr = "UCS CHECKER TOTAL: $errnone OK";
  if($errok) {
    $errstr = "$errstr, $errok ERROR$pl[0] OK [ignore]";
  }
  if($errbad) {
    $errstr = "$errstr, $errbad ERROR$pl[1] BAD [$msg]";
  }
  printflush(STDERR,"$errstr.\n");
  printflush(STDERR,"UCS CHECKER FINISHED ",`date`);
  return $errbad;
}

if($#ARGV != 1) {
  printflush(STDERR, "Usage: $0 console email\@for.errors\n");
  exit(1);
}
($dummy, $dummy, $hour) = localtime(time());
open(STDERR, ">>$ARGV[0]") || die("Can\'t open console ($ARGV[0]): $!\n");
open(STDOUT, ">$DIR/checker.log.$hour") ||
	die("Can\'t open logfile ($DIR/checker.log.$hour): $!\n");
undef $dummy;
$res = run("wait");
if(!$res) {
  exit 0;
}
open(LOG, "<$DIR/checker.log.$hour");
open(STDOUT, "|-") || exec "/usr/bin/Mail","-s","Checker: Errors Occurred","$ARGV[1]";
print(STDOUT "Initial run:\n");
while(<LOG>) {
  print(STDOUT $_);
}
close(LOG);
printflush(STDERR,"UCS CHECKER: Errors occurred, wait 60 seconds for re-run.\n");
sleep(60);
print(STDOUT "\n\nErrors occurred, re-run:\n");
$res = run("report");
if(!$res) {
  printflush(STDERR,"UCS CHECKER: No problems remain to report.\n");
  print(STDOUT "\nNo errors remain.\n");
} else {
  printflush(STDERR,"UCS CHECKER: Now report problems.\n");
  printflush(STDERR,&checkerconfig'contactinfo("UCS CHECKER:"));
  @s = ( ($res == 1) ? "" : "s", ($res == 1) ? "s" : "" );
  print(STDOUT "\n$res error$s[0] remain$s[1].\n");
}
